package cz.cvut.kbss.wpa.lab3.calculator;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

public class FormulaParser {

    /**
     * Parser the given string and returns an abstract representation of the
     * mathematical formula it represents.
     *
     * @param text The formula as a string
     * @return Evaluable formula
     */
    public Formula parse(String text) {
        List<String> tokens;
        try {
            tokens = tokenize(text);
        } catch (IOException e) {
            throw new IllegalArgumentException("Unable to parse formula " + text, e);
        }
        return generateFormula(tokens);
    }

    private List<String> tokenize(String formula) throws IOException {
        final StreamTokenizer tokenizer = new StreamTokenizer(new StringReader(formula));
        final List<String> tokens = new ArrayList<>();
        while (tokenizer.nextToken() != StreamTokenizer.TT_EOF) {
            switch (tokenizer.ttype) {
                case StreamTokenizer.TT_NUMBER:
                    tokens.add(String.valueOf(tokenizer.nval));
                    break;
                default:  // operator
                    tokens.add(String.valueOf((char) tokenizer.ttype));
            }
        }
        return tokens;
    }

    private Formula generateFormula(List<String> tokens) throws NumberFormatException {
        Stack<Token> stack = new Stack<>();
        for (String token : tokens) {
            if (isNumber(token)) {
                int num = (int) Double.parseDouble(token);
                Number n = new Number(num);
                if (stack.isEmpty()) {
                    stack.push(n);
                } else {
                    Operator op = (Operator) stack.pop();
                    op.setOperandTwo(n);
                    stack.push(op);
                }
            }
        }
        return new Formula(stack.pop());
    }

    private boolean isNumber(String s) {
        try {
            Double.parseDouble(s);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }
}
